import { jsx } from '@emotion/core';
import { useState, forwardRef, useRef, Fragment, useEffect, useContext, createContext } from 'react';
import { BaseAnchor } from '../BaseAnchor/BaseAnchor.esm.js';
import _pt from 'prop-types';
import { MdArrowForward } from '../../node_modules/react-icons/md/index.esm.esm.js';
import { AnchorButton } from '../AnchorButton/AnchorButton.esm.js';
import { Link } from 'gatsby';
import { LinkButton } from '../LinkButton/LinkButton.esm.js';
import useOnClickOutside from '../../utils/hooks/useOnClickOutside.esm.js';
import baseStyles from './BaseNavigation.styles.esm.js';
import { visuallyHiddenCss } from '../../stylesheets/a11y.esm.js';

const BaseNavigationContext = /*#__PURE__*/createContext({});
const BaseNavigation = ({
  items = [],
  secondaryItems = [],
  children,
  isInverted = false,
  mobileNavMediaQuery = `@media (max-width: 1065px)`,
  isMobileNavOpen: customIsMobileNavOpen,
  setIsMobileNavOpen: customSetIsMobileNavOpen,
  // override base components
  Hamburger = BaseNavigationHamburger,
  HamburgerIcon = BaseNavigationHamburgerIcon,
  Nav = BaseNavigationNav,
  List = BaseNavigationList,
  Item = BaseNavigationListItem,
  ItemLink = BaseNavigationItemLink,
  ItemAnchor = BaseNavigationItemAnchor,
  Dropdown = BaseNavigationDropdown,
  DropdownItem = BaseNavigationDropdownItem,
  DropdownToggle = BaseNavigationDropdownToggle,
  LinkButton = BaseNavigationLinkButton,
  AnchorButton = BaseNavigationAnchorButton,
  ...rest
}) => {
  const [internalIsMobileNavOpen, internalSetIsMobileNavOpen] = useState(false);
  const [dropdownOffsets, setDropdownOffsets] = useState({});
  let isMobileNavOpen = internalIsMobileNavOpen;
  let setIsMobileNavOpen = internalSetIsMobileNavOpen;

  if (customIsMobileNavOpen !== undefined && customSetIsMobileNavOpen !== undefined) {
    isMobileNavOpen = customIsMobileNavOpen;
    setIsMobileNavOpen = customSetIsMobileNavOpen;
  }

  const value = {
    items,
    secondaryItems,
    rootChildren: children,
    isInverted,
    mobileNavMediaQuery,
    isMobileNavOpen,
    setIsMobileNavOpen,
    dropdownOffsets,
    setDropdownOffsets,
    components: {
      Hamburger,
      HamburgerIcon,
      Nav,
      List,
      Item,
      ItemLink,
      ItemAnchor,
      Dropdown,
      DropdownItem,
      DropdownToggle,
      LinkButton,
      AnchorButton
    }
  };
  return jsx(BaseNavigationContext.Provider, {
    value: value
  }, jsx("div", {
    css: baseStyles.navigation.default,
    ...rest
  }, jsx(Hamburger, null), jsx(Nav, null)));
};
BaseNavigation.propTypes = {
  items: _pt.arrayOf(_pt.oneOfType([_pt.shape({
    name: _pt.string.isRequired,
    linkTo: _pt.string.isRequired
  }), _pt.shape({
    items: _pt.arrayOf(_pt.shape({
      name: _pt.string.isRequired,
      linkTo: _pt.string.isRequired
    }))
  })])),
  secondaryItems: _pt.arrayOf(_pt.oneOfType([_pt.shape({
    name: _pt.string.isRequired,
    linkTo: _pt.string.isRequired
  }), _pt.shape({
    items: _pt.arrayOf(_pt.shape({
      name: _pt.string.isRequired,
      linkTo: _pt.string.isRequired
    }))
  })])),
  isInverted: _pt.bool,
  mobileNavMediaQuery: _pt.string,
  isMobileNavOpen: _pt.bool,
  setIsMobileNavOpen: _pt.func
};
const BaseNavigationHamburger = /*#__PURE__*/forwardRef(function BaseNavigationHamburger(props, ref) {
  const {
    mobileNavMediaQuery,
    isMobileNavOpen,
    setIsMobileNavOpen,
    components: {
      HamburgerIcon
    }
  } = BaseNavigation.useNavigationContext();
  return jsx("button", {
    onClick: () => {
      setIsMobileNavOpen(!isMobileNavOpen);
    },
    "aria-expanded": isMobileNavOpen,
    css: { ...baseStyles.hamburger.default,
      [mobileNavMediaQuery]: baseStyles.hamburger.mobile
    },
    ...props,
    "aria-label": `${isMobileNavOpen ? "Close" : "Open"} the menu`,
    ref: ref
  }, jsx(HamburgerIcon, null));
});
BaseNavigation.Hamburger = BaseNavigationHamburger;
function BaseNavigationHamburgerIcon(props) {
  const {
    isInverted,
    isMobileNavOpen
  } = BaseNavigation.useNavigationContext();
  return jsx("div", {
    className: isMobileNavOpen ? `active` : ``,
    css: baseStyles.hamburgerIcon(isInverted),
    ...props
  });
}
BaseNavigation.HamburgerIcon = BaseNavigationHamburgerIcon;
function BaseNavigationNav(props) {
  const {
    mobileNavMediaQuery,
    isMobileNavOpen,
    components: {
      List
    }
  } = BaseNavigation.useNavigationContext();
  return jsx("nav", {
    css: theme => ({
      [mobileNavMediaQuery]: baseStyles.nav.mobile(isMobileNavOpen)(theme)
    }),
    ...props
  }, jsx(List, null));
}
BaseNavigation.Nav = BaseNavigationNav;
function BaseNavigationList(props) {
  const {
    items,
    secondaryItems,
    rootChildren,
    components: {
      Item
    }
  } = useBaseNavigationContext();
  return jsx("div", {
    css: baseStyles.list.wrapper
  }, jsx("ul", {
    css: [baseStyles.list.side, baseStyles.list.leftSide],
    ...props
  }, items.length > 0 && items.map(item => jsx(Item, {
    key: item.name,
    item: item
  })), jsx("li", {
    role: "separator",
    css: baseStyles.list.spacer
  }), secondaryItems.length > 0 && secondaryItems.map(item => jsx(Item, {
    key: item.name,
    item: item
  })), rootChildren && rootChildren));
}
BaseNavigation.List = BaseNavigationList;
function BaseNavigationListItem({
  item,
  children,
  ...rest
}) {
  const [isDropdownOpen, toggleDropdown] = useState(false);
  const dropdownItems = item.items || [];
  const dropdownChildren = children || false;
  const itemHasDropdown = dropdownItems.length > 0 || dropdownChildren;
  const ref = useRef(null); // Call hook passing in the ref and a function to call on outside click

  useOnClickOutside(ref, () => {
    if (itemHasDropdown) {
      toggleDropdown(false);
    }
  });
  const {
    isInverted,
    components: {
      ItemLink,
      DropdownToggle,
      Dropdown
    }
  } = BaseNavigation.useNavigationContext();
  return jsx("li", {
    ref: ref,
    css: baseStyles.item(isInverted),
    ...rest
  }, jsx(ItemLink, {
    item: item
  }), itemHasDropdown && jsx(Fragment, null, jsx(DropdownToggle, {
    item: item,
    isDropdownOpen: isDropdownOpen,
    toggleDropdown: toggleDropdown
  }), jsx(Dropdown, {
    item: item,
    isDropdownOpen: isDropdownOpen,
    toggleDropdown: toggleDropdown,
    dropdownItems: item.items,
    dropdownChildren: dropdownChildren
  })));
}
BaseNavigationListItem.propTypes = {
  item: _pt.oneOfType([_pt.shape({
    name: _pt.string.isRequired,
    linkTo: _pt.string.isRequired
  }), _pt.shape({
    items: _pt.arrayOf(_pt.shape({
      name: _pt.string.isRequired,
      linkTo: _pt.string.isRequired
    }))
  })]).isRequired
};
BaseNavigation.Item = BaseNavigationListItem;
function BaseNavigationItemAnchor({
  item,
  ...rest
}) {
  return jsx(BaseAnchor, {
    href: item.linkTo,
    ...rest
  }, jsx("span", null, item.name));
}
BaseNavigationItemAnchor.propTypes = {
  item: _pt.shape({
    name: _pt.string.isRequired,
    linkTo: _pt.string.isRequired
  }).isRequired
};
BaseNavigation.ItemAnchor = BaseNavigationItemAnchor;
function BaseNavigationItemLink({
  item,
  to,
  ...rest
}) {
  const {
    setIsMobileNavOpen
  } = BaseNavigation.useNavigationContext();
  return jsx(Link, {
    activeClassName: "nav-item-active",
    to: item.linkTo || to,
    onClick: () => setIsMobileNavOpen(false),
    ...rest
  }, jsx("span", null, item.name));
}
BaseNavigationItemLink.propTypes = {
  item: _pt.shape({
    name: _pt.string.isRequired,
    linkTo: _pt.string.isRequired
  }).isRequired
};
BaseNavigation.ItemLink = BaseNavigationItemLink;
const BaseNavigationDropdownToggle = /*#__PURE__*/forwardRef(function BaseNavigationDropdownToggle({
  item,
  isDropdownOpen,
  toggleDropdown,
  ...rest
}, ref) {
  const {
    isInverted
  } = BaseNavigation.useNavigationContext();
  return jsx("button", {
    ref: ref,
    "aria-expanded": !!isDropdownOpen,
    "aria-controls": getDropdownId(item.name),
    onClick: () => {
      toggleDropdown(!isDropdownOpen);
    },
    css: baseStyles.dropdownToggle(isInverted),
    ...rest
  }, jsx("span", {
    "aria-hidden": "true"
  }, "\u2228"), jsx("span", {
    css: visuallyHiddenCss
  }, `${item.name} Menu`));
});
BaseNavigationDropdownToggle.propTypes = {
  item: _pt.shape({
    name: _pt.string.isRequired,
    linkTo: _pt.string.isRequired
  }).isRequired,
  isDropdownOpen: _pt.bool.isRequired,
  toggleDropdown: _pt.func.isRequired
};
BaseNavigation.DropdownToggle = BaseNavigationDropdownToggle;
function BaseNavigationDropdown({
  item,
  isDropdownOpen,
  toggleDropdown,
  dropdownItems = [],
  dropdownChildren = false,
  dropdownListClassName,
  ...rest
}) {
  const dropdownRef = useRef(null);
  const {
    components: {
      DropdownItem
    },
    setDropdownOffsets,
    mobileNavMediaQuery
  } = BaseNavigation.useNavigationContext();
  const [isMeasured, setIsMeasured] = useState(false);
  const [windowWidth, setWindowWidth] = useState(0);
  const VIEWPORT_FIT_MARGIN = 20;
  useEffect(() => {
    setIsMeasured(true);
    setWindowWidth(window.innerWidth || document.documentElement.clientWidth);
  }, []);
  useEffect(() => {
    if (dropdownRef.current && isMeasured) {
      const {
        left,
        right
      } = dropdownRef.current.getBoundingClientRect();
      setIsMeasured(false);
      const leftFit = left >= VIEWPORT_FIT_MARGIN;
      const rightFit = right <= windowWidth - VIEWPORT_FIT_MARGIN;
      const offset = !leftFit ? (left - VIEWPORT_FIT_MARGIN) * -1 : !rightFit ? windowWidth - (right + VIEWPORT_FIT_MARGIN) : 0;
      setDropdownOffsets(state => ({ ...state,
        [item.name]: offset
      }));
    }
  }, [isMeasured]);
  return jsx("div", {
    ref: dropdownRef,
    css: baseStyles.dropdown(isDropdownOpen, isMeasured, mobileNavMediaQuery) // id to associate with aria-controls on BaseNavigation.Item
    ,
    id: getDropdownId(item.name),
    onKeyDown: e => {
      // handle closing dropdown on `esc`
      if (e.keyCode === 27) {
        toggleDropdown(false);
      }

      return;
    },
    ...rest
  }, jsx("ul", {
    css: baseStyles.dropdownList(),
    className: dropdownListClassName
  }, dropdownItems.length > 0 && dropdownItems.map((item, index) => jsx(DropdownItem, {
    key: `${index}-${item.name}`,
    item: item
  })), dropdownChildren && dropdownChildren));
}
BaseNavigationDropdown.propTypes = {
  item: _pt.shape({
    name: _pt.string.isRequired,
    linkTo: _pt.string.isRequired
  }).isRequired,
  isDropdownOpen: _pt.bool.isRequired,
  toggleDropdown: _pt.func.isRequired,
  dropdownItems: _pt.arrayOf(_pt.shape({
    name: _pt.string.isRequired,
    linkTo: _pt.string.isRequired
  })),
  dropdownChildren: _pt.node,
  dropdownListClassName: _pt.string
};
BaseNavigation.Dropdown = BaseNavigationDropdown;
function BaseNavigationDropdownItem({
  item: {
    name,
    linkTo
  },
  ...rest
}) {
  const {
    setIsMobileNavOpen
  } = BaseNavigation.useNavigationContext();
  return jsx("li", { ...rest
  }, jsx(Link, {
    activeClassName: "nav-item-active",
    to: linkTo,
    onClick: () => setIsMobileNavOpen(false)
  }, name));
}
BaseNavigationDropdownItem.propTypes = {
  item: _pt.shape({
    name: _pt.string.isRequired,
    linkTo: _pt.string.isRequired
  }).isRequired
};
BaseNavigation.DropdownItem = BaseNavigationDropdownItem;
function BaseNavigationLinkButton({
  linkTo,
  icon = true,
  size = `M`,
  children,
  to,
  ...rest
}) {
  const {
    isInverted
  } = BaseNavigation.useNavigationContext();
  return jsx(LinkButton, {
    to: linkTo || to,
    size: size,
    css: baseStyles.button(isInverted),
    rightIcon: icon ? jsx(MdArrowForward, null) : undefined,
    ...rest
  }, children);
}
BaseNavigationLinkButton.propTypes = {
  icon: _pt.bool,
  linkTo: _pt.string
};
BaseNavigation.LinkButton = BaseNavigationLinkButton;
const BaseNavigationAnchorButton = /*#__PURE__*/forwardRef(function BaseNavigationAnchorButton({
  linkTo,
  icon = true,
  size = `M`,
  children,
  ...rest
}, ref) {
  const {
    isInverted
  } = BaseNavigation.useNavigationContext();
  return jsx(AnchorButton, {
    ref: ref,
    href: linkTo,
    size: size,
    css: baseStyles.button(isInverted),
    rightIcon: icon ? jsx(MdArrowForward, null) : undefined,
    ...rest
  }, children);
});
BaseNavigationAnchorButton.propTypes = {
  icon: _pt.bool,
  linkTo: _pt.string
};
BaseNavigation.AnchorButton = BaseNavigationAnchorButton;
function useBaseNavigationContext() {
  const context = useContext(BaseNavigationContext);

  if (!context) {
    throw new Error(`BaseNavigation compound components cannot be rendered outside the BaseNavigation component`);
  }

  return context;
}
BaseNavigation.useNavigationContext = useBaseNavigationContext;

function getDropdownId(itemName) {
  // Strip item name (i.e. label) from special characters
  return `${itemName.replace(/[^\w]/gi, "-")}-dropdown`;
}

export { BaseNavigation, BaseNavigationAnchorButton, BaseNavigationDropdown, BaseNavigationDropdownItem, BaseNavigationDropdownToggle, BaseNavigationHamburgerIcon, BaseNavigationItemAnchor, BaseNavigationItemLink, BaseNavigationLinkButton, BaseNavigationList, BaseNavigationListItem, BaseNavigationNav, useBaseNavigationContext };
//# sourceMappingURL=BaseNavigation.esm.js.map
