import { jsx } from '@emotion/core';
import { Fragment } from 'react';
import _pt from 'prop-types';
import { MdCheck, MdClose } from '../../node_modules/react-icons/md/index.esm.esm.js';
import { BuildActivityStatus, BuildActivityType } from './types.esm.js';
import { BuildLogEntrySkeleton } from './BuildLogEntrySkeleton.esm.js';
import { ProgressBar } from '../ProgressBar/ProgressBar.esm.js';
import InProgressIcon from '../icons/InProgressIcon.esm.js';

const durationCellCss = theme => ({
  display: `grid`,
  alignItems: `center`,
  fontSize: theme.fontSizes[1],
  [theme.mediaQueries.tablet]: {
    gridTemplateColumns: `50% 50%`
  }
});

const durationCss = theme => ({
  textAlign: `end`,
  fontSize: theme.fontSizes[0]
});

const hiddenOnMobileCss = theme => ({
  display: `none`,
  padding: `0 ${theme.space[4]}`,
  [theme.mediaQueries.tablet]: {
    display: `block`
  }
});

const statusTextCss = theme => ({
  color: theme.colors.grey[50],
  fontSize: theme.fontSizes[0],
  marginTop: theme.space[3]
});

const statusColorLookup = theme => ({
  [BuildActivityStatus.Failed]: theme.colors.red[70],
  [BuildActivityStatus.InProgress]: theme.colors.blue[80],
  [BuildActivityStatus.NotStarted]: theme.colors.grey[50],
  [BuildActivityStatus.Success]: theme.colors.grey[50],
  [BuildActivityStatus.Interrupted]: theme.colors.red[70]
});

function BuildActivityEntry({
  activity,
  ...standardLogData
}) {
  const {
    duration,
    current,
    total,
    type,
    status
  } = activity;
  const isSuccessfulActivity = status === BuildActivityStatus.Success;
  const isFailedActivity = [BuildActivityStatus.Interrupted, BuildActivityStatus.Failed].includes(status);
  /*
   * Normalize total in case it's not updated properly when activity succeeds
   */

  const normalizedTotal = isSuccessfulActivity ? current : total;
  const showProgressBar = type === BuildActivityType.Progress && !isSuccessfulActivity && !isFailedActivity;

  const baseCss = theme => ({
    color: statusColorLookup(theme)[status]
  });

  const progressCss = theme => [hiddenOnMobileCss(theme), {
    textAlign: `right`,
    fontSize: theme.fontSizes[0],
    visibility: isFailedActivity ? `hidden` : `visible`
  }];

  const message = activity.message || activity.name || standardLogData.message; // Often statusText is the same as the message

  const showStatusText = activity.statusText && message !== activity.statusText;
  return jsx(BuildLogEntrySkeleton, {
    icon: jsx(ActivityStatusIcon, {
      activityId: activity.id,
      status: status
    }),
    css: baseCss
  }, jsx("div", {
    id: `activity__message--${activity.id}`
  }, message), jsx("div", {
    css: hiddenOnMobileCss
  }, showProgressBar && jsx(ProgressBar, {
    value: current || 0,
    max: normalizedTotal || 0,
    getProgressColor: theme => theme.colors.blue[50],
    "aria-describedby": `activity__message--${activity.id}`
  })), jsx("div", {
    css: durationCellCss
  }, jsx("span", {
    css: progressCss
  }, formatProgress(current, normalizedTotal)), typeof duration === `number` && jsx("span", {
    css: durationCss
  }, formatDuration(duration))), showStatusText && jsx(Fragment, null, jsx("div", null), jsx("div", {
    css: statusTextCss
  }, activity.statusText)));
}

const formatDuration = duration => `${duration.toFixed(2)}s`;

const formatProgress = (current, total) => [current, typeof total === "number" && typeof current === "number" && total < current ? current : total].filter(part => typeof part === `number`).join(` / `);

function ActivityStatusIcon({
  activityId,
  status
}) {
  const statusVariant = status ? activityEntryStatusVariants[status] : null;

  if (!statusVariant) {
    return null;
  }

  return jsx(statusVariant.Icon, {
    "aria-label": statusVariant.label,
    css: statusVariant.styles,
    id: `statusIcon--${activityId}`
  });
}

ActivityStatusIcon.propTypes = {
  activityId: _pt.string.isRequired
};
const activityEntryStatusVariants = {
  [BuildActivityStatus.Success]: {
    label: `Completed`,
    Icon: MdCheck,
    styles: theme => ({
      color: theme.colors.grey[40]
    })
  },
  [BuildActivityStatus.Interrupted]: {
    label: `Failed`,
    Icon: MdClose,
    styles: theme => ({
      color: theme.colors.red[60]
    })
  },
  [BuildActivityStatus.Failed]: {
    label: `Failed`,
    Icon: MdClose,
    styles: theme => ({
      color: theme.colors.red[60]
    })
  },
  [BuildActivityStatus.InProgress]: {
    label: `In Progress`,
    Icon: ActivityInProgressIcon,
    styles: theme => ({
      color: theme.colors.blue[80]
    })
  }
};

function ActivityInProgressIcon(props) {
  return jsx(InProgressIcon, { ...props,
    size: "inherit"
  });
}

export { BuildActivityEntry };
//# sourceMappingURL=BuildActivityEntry.esm.js.map
