import { jsx } from '@emotion/core';
import { useRef, forwardRef, useContext, createContext } from 'react';
import { MdSearch, MdArrowDropDown, MdDone } from '../../node_modules/react-icons/md/index.esm.esm.js';
import { visuallyHiddenCss } from '../../stylesheets/a11y.esm.js';
import { warn } from '../../utils/maintenance/warn.esm.js';
import { Combobox as Combobox$1, ComboboxInput as ComboboxInput$1, ComboboxPopover as ComboboxPopover$1, ComboboxList as ComboboxList$1, ComboboxOption as ComboboxOption$1, ComboboxOptionText as ComboboxOptionText$1, ComboboxButton as ComboboxButton$1, useComboboxContext as useComboboxContext$1 } from '@reach/combobox';
import '../../node_modules/@reach/utils/dist/utils.esm.esm.js';
import { comboboxCss, searchIconCss, inputCss, inputWithSelectedValueCss, inputWithToggleButtonCss, selectedValueCss, toggleButtonCss, popoverCss, listCss, optionCss, selectedOptionIconCss } from './Combobox.styles.esm.js';
import { DisableReachStyleCheck } from '../../utils/helpers/DisableReachStyleCheck.esm.js';

/** @jsx jsx */
const ComboboxCustomContext = /*#__PURE__*/createContext({
  listRef: {
    current: null
  }
});

function useComboboxCustomContext() {
  return useContext(ComboboxCustomContext);
}

function Combobox(props) {
  const listRef = useRef(null);
  return jsx(ComboboxCustomContext.Provider, {
    value: {
      listRef
    }
  }, jsx(DisableReachStyleCheck, {
    reachComponent: "combobox"
  }), jsx(Combobox$1, {
    openOnFocus: true,
    css: comboboxCss,
    ...props
  }));
}
const ComboboxInput = /*#__PURE__*/forwardRef(function ComboboxInput({
  selectedOptionLabel,
  hasError,
  showToggleButton,
  toggleButtonAriaLabel = "Show options",
  ...delegated
}, ref) {
  const {
    listRef
  } = useComboboxCustomContext();
  /**
   * This handler allows to scroll list of options along with keyboard navigation
   *
   * This solution has been suggested in one of the replies:
   * https://github.com/reach/reach-ui/issues/357#issuecomment-575849548
   */

  const onKeyDown = event => {
    if (event.isDefaultPrevented()) {
      return;
    }

    const container = listRef.current;

    if (!container) {
      return;
    } // According to the original Github comment, using "requestAnimationFrame" makes
    // scrolling work when navigating from last item to first item and vice versa


    window.requestAnimationFrame(() => {
      const element = container.querySelector("[aria-selected=true]");

      if (!element) {
        return;
      }

      const top = element.offsetTop - container.scrollTop;
      const bottom = container.scrollTop + container.clientHeight - (element.offsetTop + element.clientHeight);
      if (bottom < 0) container.scrollTop -= bottom;
      if (top < 0) container.scrollTop += top;
    });
  };

  let showSelectedOptionLabel = !!selectedOptionLabel;

  if (delegated.value === selectedOptionLabel) {
    showSelectedOptionLabel = false;
  }

  return jsx("div", {
    css: {
      position: "relative"
    }
  }, jsx(MdSearch, {
    css: searchIconCss,
    "aria-hidden": true
  }), jsx(ComboboxInput$1, {
    ref: ref,
    selectOnClick: true,
    onKeyDown: onKeyDown,
    css: theme => [inputCss(theme), showSelectedOptionLabel && inputWithSelectedValueCss(theme), showToggleButton && inputWithToggleButtonCss(theme)],
    "aria-invalid": hasError,
    ...delegated
  }), !!selectedOptionLabel && jsx("span", {
    "aria-hidden": true,
    css: selectedValueCss
  }, selectedOptionLabel), showToggleButton && jsx(ComboboxButton, {
    css: toggleButtonCss
  }, jsx("span", {
    css: visuallyHiddenCss
  }, toggleButtonAriaLabel), jsx(MdArrowDropDown, {
    "aria-hidden": true
  })));
});
const ComboboxPopover = /*#__PURE__*/forwardRef(function ComboboxPopover(props, ref) {
  return jsx(ComboboxPopover$1, {
    ref: ref,
    portal: true,
    css: popoverCss,
    ...props
  });
});
function ComboboxList(props) {
  const {
    listRef
  } = useComboboxCustomContext();

  if (process.env.NODE_ENV === `development`) {
    const hasAccessibleName = Boolean(props["aria-label"] || props["aria-labelledby"]);

    if (!hasAccessibleName) {
      warn(`<ComboboxList /> is missing one of the required props: "aria-label", "aria-labelledby"`);
    }
  }

  return jsx(ComboboxList$1, {
    ref: listRef,
    persistSelection: true,
    css: listCss,
    ...props
  });
}
const ComboboxOption = /*#__PURE__*/forwardRef(function ComboboxOption({
  selected,
  selectedAriaLabel = "currently selected:",
  highlightMatches = true,
  value,
  children,
  ...delegated
}, ref) {
  return jsx(ComboboxOption$1, {
    ref: ref,
    value: value,
    css: optionCss(highlightMatches),
    ...delegated
  }, selected && jsx(MdDone, {
    css: selectedOptionIconCss,
    "aria-label": selectedAriaLabel
  }), children || (highlightMatches ? jsx(ComboboxOptionText, null) : value));
});
function ComboboxOptionText(props) {
  return jsx(ComboboxOptionText$1, { ...props
  });
}
function ComboboxButton(props) {
  // According to WAI-ARIA authoring practices, combobox button should be excluded from the tab sequence
  // https://www.w3.org/TR/wai-aria-practices-1.1/#keyboard-interaction-6
  return jsx(ComboboxButton$1, {
    tabIndex: -1,
    ...props,
    type: "button"
  });
}
function useComboboxContext() {
  return useComboboxContext$1();
}

export { Combobox, ComboboxButton, ComboboxInput, ComboboxList, ComboboxOption, ComboboxOptionText, ComboboxPopover, useComboboxContext };
//# sourceMappingURL=Combobox.esm.js.map
