import { jsx, ClassNames } from '@emotion/core';
import { forwardRef } from 'react';
import { BaseNavigation } from '../BaseNavigation/BaseNavigation.esm.js';
import { navigationBaseCss, hamburgerIconCss, navCss, listCss, itemCss, dropdownCss, dropdownListCss, dropdownToggleCss, dropdownItemCss, itemLinkCss, buttonItemCss } from './Navigation.styles.esm.js';

/** @jsx jsx */
function Navigation({
  items,
  isInverted = false,
  mobileNavMediaQuery = `@media (max-width: 1065px)`,
  ...delegated
}) {
  return jsx(BaseNavigation, {
    items: items,
    isInverted: isInverted,
    mobileNavMediaQuery: mobileNavMediaQuery // override base components
    ,
    Hamburger: Navigation.Hamburger,
    HamburgerIcon: Navigation.HamburgerIcon,
    Nav: Navigation.Nav,
    List: Navigation.List,
    Item: Navigation.Item,
    ItemLink: Navigation.ItemLink,
    Dropdown: Navigation.Dropdown,
    DropdownItem: Navigation.DropdownItem,
    DropdownToggle: Navigation.DropdownToggle,
    css: navigationBaseCss,
    ...delegated
  });
}

function useNavigationContext() {
  return BaseNavigation.useNavigationContext();
}

Navigation.useNavigationContext = useNavigationContext;
const NavigationHamburger = /*#__PURE__*/forwardRef(function NavigationHamburger(delegated, ref) {
  return jsx(BaseNavigation.Hamburger, { ...delegated,
    ref: ref
  });
});
Navigation.Hamburger = NavigationHamburger;

function NavigationHamburgerIcon(delegated) {
  const {
    isMobileNavOpen
  } = Navigation.useNavigationContext();
  return jsx(BaseNavigation.HamburgerIcon, {
    css: hamburgerIconCss,
    className: isMobileNavOpen ? `active` : ``,
    ...delegated
  });
}

Navigation.HamburgerIcon = NavigationHamburgerIcon;

function NavigationNav(delegated) {
  const {
    mobileNavMediaQuery,
    isMobileNavOpen
  } = Navigation.useNavigationContext();
  return jsx(BaseNavigation.Nav, {
    css: navCss(mobileNavMediaQuery, isMobileNavOpen),
    ...delegated
  });
}

Navigation.Nav = NavigationNav;

function NavigationList(delegated) {
  const {
    mobileNavMediaQuery
  } = Navigation.useNavigationContext();
  return jsx(BaseNavigation.List, {
    css: listCss(mobileNavMediaQuery),
    ...delegated
  });
}

Navigation.List = NavigationList;

function NavigationSpacer(delegated) {
  return jsx("li", {
    css: {
      flex: 1
    },
    ...delegated,
    role: "separator"
  });
}

Navigation.Spacer = NavigationSpacer;

function NavigationItem(delegated) {
  const {
    mobileNavMediaQuery
  } = Navigation.useNavigationContext();
  return jsx(BaseNavigation.Item, {
    css: itemCss(mobileNavMediaQuery),
    ...delegated
  });
}

Navigation.Item = NavigationItem;

function NavigationItemLink(delegated) {
  const {
    isInverted,
    mobileNavMediaQuery
  } = Navigation.useNavigationContext();
  const commonCss = itemLinkCss(mobileNavMediaQuery, isInverted);

  if (isExternalLinkItem(delegated)) {
    return jsx(BaseNavigation.ItemAnchor, {
      css: commonCss,
      ...delegated
    });
  }

  return jsx(BaseNavigation.ItemLink, {
    css: commonCss,
    ...delegated
  });
}

Navigation.ItemLink = NavigationItemLink;

function isExternalLinkItem(props) {
  return isExternalLink(props.item.linkTo);
}

function NavigationDropdown(delegated) {
  const {
    mobileNavMediaQuery,
    dropdownOffsets
  } = Navigation.useNavigationContext();
  const itemOffset = dropdownOffsets[delegated.item.name];
  const offset = itemOffset ? `${itemOffset}px` : `0px`;
  return jsx(ClassNames, null, ({
    css,
    theme
  }) => jsx(BaseNavigation.Dropdown, {
    css: dropdownCss(mobileNavMediaQuery, offset),
    dropdownListClassName: css(dropdownListCss(theme, mobileNavMediaQuery)),
    ...delegated
  }));
}

Navigation.Dropdown = NavigationDropdown;
const NavigationDropdownToggle = /*#__PURE__*/forwardRef(function NavigationDropdownToggle(delegated, ref) {
  const {
    mobileNavMediaQuery
  } = Navigation.useNavigationContext();
  return jsx(BaseNavigation.DropdownToggle, {
    css: dropdownToggleCss(mobileNavMediaQuery),
    ...delegated,
    ref: ref
  });
});
Navigation.DropdownToggle = NavigationDropdownToggle;

function NavigationDropdownItem(delegated) {
  const {
    mobileNavMediaQuery
  } = Navigation.useNavigationContext();
  return jsx(BaseNavigation.DropdownItem, {
    css: dropdownItemCss(mobileNavMediaQuery),
    ...delegated
  });
}

Navigation.DropdownItem = NavigationDropdownItem;

function NavigationButton({
  linkTo,
  ...delegated
}) {
  const {
    mobileNavMediaQuery
  } = Navigation.useNavigationContext();
  const commonCss = buttonItemCss(mobileNavMediaQuery);

  if (isExternalButtonLink(linkTo)) {
    return jsx("li", {
      css: commonCss
    }, jsx(BaseNavigation.AnchorButton, {
      href: linkTo,
      ...delegated
    }));
  }

  return jsx("li", {
    css: commonCss
  }, jsx(BaseNavigation.LinkButton, {
    linkTo: linkTo,
    ...delegated
  }));
}

Navigation.Button = NavigationButton;

function isExternalButtonLink(linkTo, _props) {
  return !linkTo || isExternalLink(linkTo);
}

const isExternalLink = linkTo => {
  return Boolean(linkTo.match(/(^http|^mailto)/i));
};

export { Navigation };
//# sourceMappingURL=Navigation.esm.js.map
