import { jsx } from '@emotion/core';
import React, { Fragment } from 'react';
import _pt from 'prop-types';
import { MdArrowForward, MdClose } from '../../node_modules/react-icons/md/index.esm.esm.js';
import { Button } from '../Button/Button.esm.js';
import { Link } from '../Link/Link.esm.js';
import { getNotificationVariantStyles, iconByTone } from './Notification.helpers.esm.js';

const NotificationContext = /*#__PURE__*/React.createContext({
  onDismiss: () => undefined,
  variant: undefined,
  tone: undefined
});

const baseCss = theme => ({
  color: theme.colors.grey[90],
  display: `flex`,
  alignItems: `flex-start`,
  justifyContent: `space-between`,
  flexWrap: `nowrap`,
  width: `100%`,
  fontSize: theme.fontSizes[1],
  lineHeight: theme.lineHeights.default
});

function Notification({
  children,
  tone = `BRAND`,
  variant = `PRIMARY`,
  content,
  contentAs = `span`,
  linkUrl,
  linkText,
  onLinkClick,
  isOpened = true,
  onDismissButtonClick,
  showDismissButton = !!onDismissButtonClick,
  dismissButtonLabel,
  Icon: CustomIcon,
  ...rest
}) {
  if (!isOpened) {
    return null;
  }

  const PresetIcon = content && iconByTone[tone];
  const Icon = CustomIcon || PresetIcon;
  return jsx(NotificationContext.Provider, {
    value: {
      onDismiss: onDismissButtonClick,
      variant,
      tone
    }
  }, jsx("div", {
    css: theme => [variant === `PRIMARY` && theme.cardStyles.frame, baseCss(theme), getNotificationVariantStyles(variant, tone)(theme)],
    ...rest
  }, content && jsx(NotificationContent, {
    as: contentAs,
    css: theme => [linkUrl && linkText && {
      marginRight: theme.space[5]
    }]
  }, Icon && jsx(Icon, {
    css: theme => [{
      marginRight: theme.space[4],
      fontSize: theme.fontSizes[4],
      color: theme.tones[tone].medium,
      flexShrink: 0,
      width: "auto",
      height: "1em"
    }, variant === `SOLID` && {
      color: theme.tones[tone].mediumInverted ? theme.tones[tone].mediumInverted : theme.colors.whiteFade[90]
    }]
  }), content), linkUrl && linkText && jsx(Link, {
    to: linkUrl,
    onClick: onLinkClick,
    css: theme => ({
      // to push <Link> to the right also when there's a
      // <NotificationDismissButton>
      marginLeft: "auto",
      color: variant === `SOLID` ? theme.colors.white : null,
      ":hover": {
        color: variant === `SOLID` ? theme.colors.whiteFade[80] : null
      }
    })
  }, linkText && jsx(Fragment, null, linkText, " ", jsx(MdArrowForward, null))), showDismissButton && jsx(NotificationDismissButton, {
    label: dismissButtonLabel
  }), children));
}
Notification.propTypes = {
  children: _pt.node,
  variant: _pt.oneOf([`PRIMARY`, `SECONDARY`, `SOLID`]),
  tone: _pt.oneOf([`BRAND`, `SUCCESS`, `DANGER`, `WARNING`, `NEUTRAL`]),
  content: _pt.node,
  linkUrl: _pt.string,
  linkText: _pt.node,
  onLinkClick: _pt.func,
  isOpened: _pt.bool,
  onDismissButtonClick: _pt.func,
  showDismissButton: _pt.bool,
  dismissButtonLabel: _pt.string,
  Icon: _pt.elementType
};

function NotificationContent({
  as: Component = `span`,
  ...rest
}) {
  return jsx(Component, {
    css: {
      display: `flex`,
      alignItems: `flex-start`
    },
    ...rest
  });
}

NotificationContent.propTypes = {
  as: _pt.oneOf(["span", "div"])
};

function NotificationDismissButton({
  label = `Close`
}) {
  const {
    onDismiss,
    variant,
    tone
  } = useNotificationContext();
  return jsx(Button, {
    css: theme => [{
      padding: `0`,
      minHeight: `auto`,
      color: theme.colors.grey[40],
      width: theme.space[5],
      marginLeft: theme.space[5],
      fontSize: theme.fontSizes[4]
    }, variant === `SOLID` && {
      color: theme.colors.whiteFade[60],
      ":hover": {
        background: "transparent",
        color: theme.colors.white
      }
    }, variant === `SOLID` && tone === `WARNING` && {
      color: theme.colors.blackFade[60],
      ":hover": {
        color: theme.colors.black
      }
    }],
    type: "button",
    onClick: onDismiss,
    variant: "GHOST",
    "aria-label": label
  }, jsx(MdClose, null));
}

function useNotificationContext() {
  const context = React.useContext(NotificationContext);

  if (!context) {
    throw new Error(`Notification compound components cannot be rendered outside the main component`);
  }

  return context;
}

export { Notification };
//# sourceMappingURL=Notification.esm.js.map
