import { jsx } from '@emotion/core';
import { useRef, useState, useEffect, Fragment } from 'react';
import _pt from 'prop-types';
import format from '../../node_modules/date-fns/esm/format/index.esm.js';

const baseCss = theme => ({
  padding: theme.space[5],
  backgroundColor: theme.colors.grey[80],
  color: theme.colors.white,
  fontFamily: theme.fonts.monospace,
  fontSize: theme.fontSizes[1],
  lineHeight: theme.lineHeights.default,
  fontVariantLigatures: `none`,
  [theme.mediaQueries.desktop]: {
    padding: theme.space[7]
  }
});

const listCss = _theme => ({
  listStyle: `none`,
  margin: 0,
  padding: 0,
  height: `100%`,
  overflowY: `auto`
});

const itemCss = theme => [{
  margin: 0,
  lineHeight: 1.8,
  color: theme.colors.grey[30],
  time: {
    whiteSpace: `nowrap`,
    marginRight: theme.space[3],
    color: theme.colors.grey[40],
    fontStyle: `italic`
  },
  p: {
    margin: 0,
    whiteSpace: `pre-wrap`,
    marginLeft: theme.space[5],
    span: {
      fontWeight: `bold`,
      "&.success": {
        color: theme.colors.green[40]
      },
      "&.info": {
        color: theme.colors.blue[40]
      },
      "&.warning": {
        color: theme.colors.orange[50]
      },
      "&.error": {
        color: theme.colors.red[10],
        background: theme.colors.red[90],
        padding: `${theme.space[1]} ${theme.space[2]}`,
        borderRadius: theme.radii[2],
        textTransform: `uppercase`,
        "&:after, &:before": {
          content: `"!"`
        }
      }
    }
  },
  [theme.mediaQueries.desktop]: {
    display: `flex`,
    p: {
      marginLeft: 0
    }
  }
}];

function formatMessage(message) {
  if (!message) {
    return;
  }

  const formated = message.replace(/(^|\n)(success|info|warning|error) /gi, (_, match1, match2) => {
    return `${match1}<span class="${match2.toLowerCase()}">${match2}</span> `;
  });
  return jsx("p", {
    dangerouslySetInnerHTML: {
      __html: formated
    }
  });
}

function RawLogs({
  logItems,
  "aria-label": ariaLabel,
  timeFormat = "HH:mm:ss aa:",
  className
}) {
  const listRef = useRef(null);
  const [autoScroll, setAutoScroll] = useState(false);
  useEffect(() => {
    if (!autoScroll || !listRef.current) {
      return;
    }

    if (listRef.current.scrollHeight > listRef.current.clientHeight) {
      listRef.current.scrollTop = listRef.current.scrollHeight;
    } else {
      listRef.current.lastChild.scrollIntoView();
    }
  });

  const handleScroll = e => {
    const list = e.target;
    const isAtEnd = list.scrollHeight - list.scrollTop === list.clientHeight;

    if (isAtEnd) {
      setAutoScroll(true);
    } else if (autoScroll && !isAtEnd) {
      setAutoScroll(false);
    }
  };

  return jsx("div", {
    role: "log",
    css: baseCss,
    className: className
  }, jsx("ol", {
    tabIndex: 0,
    ref: listRef,
    "aria-label": ariaLabel,
    css: listCss,
    onScroll: handleScroll
  }, logItems.map(({
    id,
    timestamp,
    message
  }, idx) => {
    let key = idx;

    if (id) {
      key = id;
    } else if (message && timestamp) {
      key = `${message}--${timestamp}`;
    }

    const timestampDate = timestamp ? new Date(timestamp) : null;
    return jsx("li", {
      key: key,
      css: itemCss
    }, timestampDate && jsx(Fragment, null, jsx("time", {
      dateTime: timestampDate.toString()
    }, format(timestampDate, timeFormat))), formatMessage(message));
  })));
}
RawLogs.propTypes = {
  logItems: _pt.arrayOf(_pt.shape({
    id: _pt.oneOfType([_pt.string, _pt.oneOf([null])]),
    message: _pt.node,
    timestamp: _pt.oneOfType([_pt.number, _pt.string]),
    sourceStream: _pt.oneOfType([_pt.oneOf(["STDERR"]), _pt.oneOf(["STDOUT"]), _pt.oneOf([null])])
  })).isRequired,
  "aria-label": _pt.string.isRequired,
  timeFormat: _pt.string,
  className: _pt.string
};

export { RawLogs };
//# sourceMappingURL=RawLogs.esm.js.map
