import { jsx } from '@emotion/core';
import { Fragment } from 'react';
import _pt from 'prop-types';
import { Link } from 'gatsby';
import { warn } from '../../utils/maintenance/warn.esm.js';

function SidebarNav({
  options,
  ...rest
}) {
  return jsx("nav", {
    "aria-label": "sidebar navigation",
    css: theme => ({
      paddingLeft: theme.space[8]
    }),
    ...rest
  }, options && jsx(SidebarNavList, null, options.map(option => {
    return jsx(SidebarNavListItem, {
      key: option.to,
      ...option
    });
  })));
}
SidebarNav.propTypes = {
  options: _pt.arrayOf(_pt.oneOfType([_pt.shape({
    label: _pt.node.isRequired,
    to: _pt.string,
    href: _pt.string,
    active: _pt.bool,
    onClick: _pt.func,
    toggled: _pt.bool
  }), _pt.shape({
    Icon: _pt.elementType,
    subItems: _pt.arrayOf(_pt.shape({
      label: _pt.node.isRequired,
      to: _pt.string,
      href: _pt.string,
      active: _pt.bool,
      onClick: _pt.func,
      toggled: _pt.bool
    }))
  })]))
};

function SidebarNavList(props) {
  return jsx("ul", {
    css: [{
      listStyle: `none`,
      margin: 0,
      padding: 0
    }],
    ...props
  });
}

const baseNavItemCss = theme => ({
  fontFamily: theme.fonts.body,
  fontSize: theme.fontSizes[1],
  fontWeight: theme.fontWeights.body,
  color: theme.colors.grey[60],
  marginBottom: `0`,
  "&:first-of-type": {
    paddingTop: `0`
  },
  "&:last-of-type": {
    paddingBottom: `0`
  }
});

const baseNavItemActiveCss = theme => ({
  color: theme.colors.purple[50],
  fontWeight: theme.fontWeights.semiBold
});

const navItemIconCss = theme => ({
  color: theme.colors.grey[40],
  fontSize: theme.fontSizes[4],
  verticalAlign: `middle`,
  position: `absolute`,
  left: `-${theme.space[8]}`
});

const navItemActiveIconCss = theme => ({
  color: theme.colors.purple[40]
});

function SidebarNavListItem({
  label,
  to,
  active,
  onClick,
  Icon,
  subItems,
  toggled,
  ...rest
}) {
  return jsx(SidebarBaseItem, {
    css: theme => [{
      position: `relative`,
      padding: `${theme.space[3]} 0`
    }],
    label: jsx(Fragment, null, Icon && jsx(Icon, {
      css: theme => [navItemIconCss(theme), active && navItemActiveIconCss(theme)],
      size: "inherit"
    }), label),
    to: to,
    className: active ? "active" : undefined,
    active: active,
    onClick: onClick,
    current: active ? "page" : undefined,
    ...rest
  }, subItems && (active ? !toggled : toggled) && jsx(SidebarNavList, {
    css: theme => ({
      paddingTop: theme.space[5],
      paddingBottom: theme.space[3]
    })
  }, subItems.map(subItem => jsx(SidebarBaseItem, {
    key: subItem.to,
    css: theme => [{
      paddingTop: theme.space[3],
      paddingRight: theme.space[5],
      paddingBottom: theme.space[3],
      paddingLeft: `calc(${theme.space[6]} + 1px)`,
      marginBottom: `0`,
      marginLeft: `calc(-${theme.space[6]} - 2px)`,
      borderLeft: `1px solid ${theme.colors.grey[30]}`
    }, subItem.active && {
      borderLeftColor: theme.colors.purple[50]
    }],
    current: subItem.active ? "location" : undefined,
    ...subItem
  }))));
}

SidebarNavListItem.propTypes = {
  label: _pt.node.isRequired,
  to: _pt.string,
  href: _pt.string,
  active: _pt.bool,
  onClick: _pt.func,
  toggled: _pt.bool,
  Icon: _pt.elementType,
  subItems: _pt.arrayOf(_pt.shape({
    label: _pt.node.isRequired,
    to: _pt.string,
    href: _pt.string,
    active: _pt.bool,
    onClick: _pt.func,
    toggled: _pt.bool
  }))
};

const itemLinkCss = theme => ({
  color: `inherit`,
  lineHeight: theme.lineHeights.dense,
  textDecoration: `none`
});

function SidebarBaseItem({
  children,
  label,
  to,
  active,
  onClick,
  current,
  href,
  ...rest
}) {
  const isOverwritten = to && href;
  const isText = !to && !href || isOverwritten;

  if (isOverwritten) {
    warn(`You can't set both 'to' and 'href' props on SidebarBaseItem at the same time! Check the '${label}' link.`, `warning`);
  }

  return jsx("li", {
    css: theme => [baseNavItemCss(theme), active && baseNavItemActiveCss(theme)],
    className: active ? "active" : "",
    ...rest
  }, to && !href && jsx(Link, {
    to: to,
    onClick: onClick,
    css: itemLinkCss,
    "aria-current": current
  }, label), !to && href && jsx("a", {
    href: href,
    onClick: onClick,
    css: itemLinkCss,
    "aria-current": current
  }, label), isText && label, children);
}

SidebarBaseItem.propTypes = {
  label: _pt.node.isRequired,
  to: _pt.string,
  href: _pt.string,
  active: _pt.bool,
  onClick: _pt.func,
  toggled: _pt.bool,
  current: _pt.oneOf(["page", "location"])
};

export default SidebarNav;
//# sourceMappingURL=SidebarNav.esm.js.map
