import { jsx } from '@emotion/core';
import { useState, useContext, Fragment, useRef, useEffect, createContext } from 'react';
import _pt from 'prop-types';

const StickyObserverContext = /*#__PURE__*/createContext({
  isStuck: false,
  lipShadowPosition: `bottom`,
  setIsStuck: () => undefined
});
// This component is a shorthand; use
//  - StickyObserverProvider,
//  - StickyObservedContainer
//  - StickyObserverSentinel
//  - StickyLipShadow
// if you need more control over DOM structure,
function StickyObserver({
  lipShadowPosition,
  children,
  ...rest
}) {
  return jsx(StickyObserverProvider, {
    lipShadowPosition: lipShadowPosition
  }, jsx(StickyObservedContainer, { ...rest
  }, children));
}
StickyObserver.propTypes = {
  lipShadowPosition: _pt.oneOf([`top`, `bottom`]).isRequired
};
function StickyObserverProvider({
  lipShadowPosition,
  children
}) {
  const [isStuck, setIsStuck] = useState(false);
  const contextValue = {
    isStuck,
    setIsStuck,
    lipShadowPosition
  };
  return jsx(StickyObserverContext.Provider, {
    value: contextValue
  }, children);
}
StickyObserverProvider.propTypes = {
  lipShadowPosition: _pt.oneOf([`top`, `bottom`]).isRequired,
  children: _pt.node.isRequired
};
function useStickyObserver() {
  return useContext(StickyObserverContext);
}
function StickyObservedContainer({
  children,
  ...rest
}) {
  const {
    lipShadowPosition
  } = useStickyObserver();
  return jsx(Fragment, null, lipShadowPosition === `bottom` && jsx(StickyObserverSentinel, null), jsx(StickyObserverSentinel, null), jsx("div", {
    css: [{
      position: "sticky",
      zIndex: 1,
      left: 0,
      width: "100%"
    }, lipShadowPosition === `top` && {
      bottom: 0
    }, lipShadowPosition === `bottom` && {
      top: 0
    }],
    ...rest
  }, lipShadowPosition === `top` && jsx(StickyLipShadow, null), children, lipShadowPosition === `bottom` && jsx(StickyLipShadow, null)), lipShadowPosition === `top` && jsx(StickyObserverSentinel, null));
}
// A "sentinel" is a hack DIV element that is rendered next to the sticky element:
//  - before the sticky element if it sticks to the top
//  - after the sticky element if it sticks to the bottom
// IntersectionObserver does not work properly for elements with position: sticky,
// and this is why this "sentinel" is needed if we want to use Intersection API to identify
// whether a sticky element has "stuck".
//
// When the sticky element gets "stuck", it pushes the sentinel away from the screen, triggering an intersection entry
// When the sticky element gets "unstuck", the sentinel appears on the screen, triggering an intersection entry
function StickyObserverSentinel(_props) {
  const {
    setIsStuck,
    lipShadowPosition
  } = useStickyObserver();
  const sentinelRef = useRef(null);
  const observerRef = useRef(null);
  useEffect(() => {
    if (!sentinelRef.current) {
      return;
    }

    observerRef.current = new IntersectionObserver(entries => {
      if (entries[0].intersectionRatio === 0 && !entries[0].isIntersecting) {
        setIsStuck(true);
      } else if (entries[0].intersectionRatio === 1) {
        setIsStuck(false);
      }
    }, {
      threshold: [0, 1]
    });
    observerRef.current.observe(sentinelRef.current);
    return () => {
      if (observerRef.current) {
        observerRef.current.disconnect();
      }
    };
  }, [lipShadowPosition, sentinelRef.current]);
  return jsx("div", {
    ref: sentinelRef,
    css: {
      height: `1px`,
      backgroundColor: `inherit`
    },
    "aria-hidden": true
  });
}
// Lip shadows MUST be rendered as children of the sticky element for proper positioning
function StickyLipShadow(_props) {
  const {
    isStuck,
    lipShadowPosition
  } = useStickyObserver();
  return jsx("div", {
    "data-sticky-lip-visible": isStuck,
    css: theme => [{
      height: 0,
      position: `relative`,
      ":before": {
        content: '""',
        display: `block`,
        position: `absolute`,
        left: 0,
        right: 0,
        height: 1,
        transition: `box-lip ${theme.transitions.default}`,
        boxShadow: `none`
      }
    }, lipShadowPosition === `top` && {
      ":before": {
        top: 0
      },
      '&[data-sticky-lip-visible="true"]:before': {
        boxShadow: getLipBoxShadow(theme, `top`)
      }
    }, lipShadowPosition === `bottom` && {
      ":before": {
        bottom: 0
      },
      '&[data-sticky-lip-visible="true"]:before': {
        boxShadow: getLipBoxShadow(theme, `bottom`)
      }
    }]
  });
}

function getLipBoxShadow(theme, shadowDirection) {
  return `
        0px ${shadowDirection === "top" ? -2 : 2}px 2px ${theme.colors.blackFade[10]},
        0px ${shadowDirection === "top" ? -4 : 4}px 4px ${theme.colors.blackFade[20]}
    `;
}

export { StickyLipShadow, StickyObservedContainer, StickyObserver, StickyObserverProvider, StickyObserverSentinel, useStickyObserver };
//# sourceMappingURL=StickyObserver.esm.js.map
