import { jsx } from '@emotion/core';
import { Fragment } from 'react';
import { visuallyHiddenCss } from '../../../stylesheets/a11y.esm.js';
import { getErrorAriaLiveAttribute, getHintId, getErrorId } from '../utils/index.esm.js';

/** @jsx jsx */

/**
 * "group" fields (radio buttons and checkbox groups) are more complicated
 * than "single" ones when it comes to aria-attributes
 *
 * Apparently, we cannot just put aria-invalid and aria-describedby to each option's input,
 * as this would result in too many screen reader announcements
 *
 * Atatching those aria- attributes to only the first option kinda works but not entirely without issues
 *
 * So instead we are going to require the actual hint and error messages and put them (visually hidden) in the group label,
 * while marking the actual visible messages as hidden from screen readers
 * For more details see this article: https://blog.tenon.io/accessible-validation-of-checkbox-and-radiobutton-groups
 */
function useAriaFormGroupField(fieldId, {
  required = false,
  error,
  hint,
  hasError = Boolean(error),
  hasHint = Boolean(hint),
  validationMode
}) {
  const hintId = getHintId(fieldId);
  const errorId = getErrorId(fieldId);
  return {
    getLegendProps: label => ({
      children: jsx(Fragment, null, label, jsx("div", {
        css: visuallyHiddenCss
      }, jsx("div", null, hint), jsx("div", null, error)))
    }),
    getOptionControlProps: optionValue => ({
      id: getGroupOptionId(fieldId, optionValue),
      required
    }),
    getOptionLabelProps: optionValue => ({
      htmlFor: getGroupOptionId(fieldId, optionValue)
    }),
    hintProps: {
      id: hintId,
      hidden: !hasHint,
      "aria-hidden": true
    },
    errorProps: {
      id: errorId,
      hidden: !hasError,
      "aria-hidden": true,
      "aria-live": getErrorAriaLiveAttribute(validationMode)
    },
    meta: {
      required
    }
  };
}

function getGroupOptionId(fieldId, optionValue) {
  return `${fieldId}__option--${optionValue}`;
}

export { useAriaFormGroupField };
//# sourceMappingURL=useAriaFormGroupField.esm.js.map
