import { jsx } from '@emotion/core';
import _pt from 'prop-types';
import { useAriaFormField } from '../../form-skeletons/hooks/useAriaFormField.esm.js';
import { useAriaFormGroupField } from '../../form-skeletons/hooks/useAriaFormGroupField.esm.js';
import { StyledLabel, FormHint, FormError, FormFieldset, FormLegend } from './styled-primitives/StyledFormElements.esm.js';
import { getFieldLayoutStyles } from '../styles/index.esm.js';

function FormFieldBlock({
  id,
  label,
  error,
  hint,
  required,
  labelSize,
  validationMode,
  layout,
  className,
  children
}) {
  const fieldData = useAriaFormField(id, {
    required: required,
    hasError: !!error,
    hasHint: !!hint,
    validationMode
  });
  return jsx(FormFieldBlockBoilerplate, {
    fieldData: fieldData,
    label: label,
    error: error,
    hint: hint,
    layout: layout,
    labelSize: labelSize,
    className: className
  }, typeof children === `function` ? children(fieldData.controlProps) : children);
}
FormFieldBlock.propTypes = {
  id: _pt.string.isRequired,
  label: _pt.node.isRequired,
  error: _pt.node,
  hint: _pt.node,
  required: _pt.bool,
  className: _pt.string,
  children: _pt.oneOfType([_pt.node, _pt.func]).isRequired
};
function FormFieldBlockBoilerplate({
  fieldData,
  children,
  label,
  error,
  hint,
  layout,
  labelSize,
  ...rest
}) {
  const {
    labelCss,
    fieldBodyCss
  } = getFieldLayoutStyles(layout);

  const finalLabelCss = theme => [labelCss(theme), {
    display: `block`
  }];

  return jsx("div", { ...rest
  }, jsx(StyledLabel, {
    required: fieldData.controlProps.required,
    labelSize: labelSize,
    ...fieldData.labelProps,
    css: finalLabelCss
  }, label), jsx("div", {
    css: fieldBodyCss
  }, children, jsx(FormHint, { ...fieldData.hintProps
  }, hint), jsx(FormError, { ...fieldData.errorProps
  }, error)));
}
/**
 * Group field organism
 *
 */

function FormGroupFieldBlock({
  id,
  label,
  error,
  hint,
  required,
  labelSize,
  validationMode,
  layout,
  className,
  children
}) {
  const fieldData = useAriaFormGroupField(id, {
    required: required,
    error,
    hint,
    validationMode
  });
  return jsx(FormGroupFieldBlockBoilerplate, {
    fieldData: fieldData,
    label: label,
    error: error,
    hint: hint,
    layout: layout,
    labelSize: labelSize,
    className: className
  }, children({
    getOptionControlProps: fieldData.getOptionControlProps,
    getOptionLabelProps: fieldData.getOptionLabelProps
  }));
}
FormGroupFieldBlock.propTypes = {
  id: _pt.string.isRequired,
  label: _pt.node.isRequired,
  error: _pt.node,
  hint: _pt.node,
  required: _pt.bool,
  className: _pt.string,
  children: _pt.func.isRequired
};
function FormGroupFieldBlockBoilerplate({
  fieldData,
  children,
  label,
  error,
  hint,
  layout,
  labelSize,
  ...rest
}) {
  const {
    labelCss,
    fieldBodyCss
  } = getFieldLayoutStyles(layout, true);

  const finalLabelCss = theme => [labelCss(theme), {
    display: `block`
  }];

  return jsx(FormFieldset, { ...rest
  }, jsx(FormLegend, {
    labelSize: labelSize,
    required: fieldData.meta.required,
    ...fieldData.getLegendProps(label),
    css: finalLabelCss
  }), jsx("div", {
    css: fieldBodyCss
  }, children, jsx(FormHint, { ...fieldData.hintProps
  }, hint), jsx(FormError, { ...fieldData.errorProps
  }, error)));
}

export { FormFieldBlock, FormFieldBlockBoilerplate, FormGroupFieldBlock, FormGroupFieldBlockBoilerplate };
//# sourceMappingURL=FormFieldBlock.esm.js.map
